@@ -3,6 +3,7 @@ import os
 import re
 import sh
 import shutil
+import yaml
 
 from enough.common.dotenough import DotEnough, Hosts
 from enough.common.host import host_factory
@@ -26,17 +27,18 @@ class Enough(object):
         self.args = self.init_args.copy()
         self.args.update(kwargs)
 
-        if 'clouds' not in self.args:
-            self.args['clouds'] = f'{self.config_dir}/inventory/group_vars/all/clouds.yml'
         self.dotenough = DotEnough(self.config_dir, self.args['domain'])
+        self.ansible = ansible_utils.Ansible(self.config_dir, self.share_dir)
+        if self.args.get('driver', 'openstack'):
+            if 'cloud' not in self.args:
+                self.args['cloud'] = self.ansible.get_global_variable('openstack_cloud')
         self.hosts = Hosts(self.config_dir)
         self.host = host_factory(self.config_dir, self.share_dir, **self.args)
         self.service = service_factory(self.config_dir, self.share_dir, **self.args)
         if self.args.get('driver', 'openstack'):
-            self.openstack = OpenStack(self.config_dir, self.args['clouds'])
-            self.heat = Heat(self.config_dir, self.args['clouds'], **self.args)
+            self.openstack = OpenStack(self.config_dir, **self.args)
+            self.heat = Heat(self.config_dir, **self.args)
         self.playbook = ansible_utils.Playbook(self.config_dir, self.share_dir)
-        self.ansible = ansible_utils.Ansible(self.config_dir, self.share_dir)
 
     def create_missings(self, hosts):
         public_key = f'{self.config_dir}/infrastructure_key.pub'
@@ -51,7 +53,7 @@ class Enough(object):
             return
         log.info('update internal network dns server with bind-host IP address')
         dns = []
-        for host in self.service.service2hosts['bind']:
+        for host in self.service.service2group['bind']:
             dns.append(self.openstack.server_ip_in_network(host, self.openstack.INTERNAL_NETWORK))
         self.openstack.subnet_update_dns(self.openstack.INTERNAL_NETWORK, *dns)
 
@@ -86,30 +88,34 @@ class Enough(object):
             sh.sudo.kill(pid)
             sh.sudo.rm('/var/run/openvpn-localhost.pid')
 
-    def clone(self, target_domain, target_clouds):
+    @staticmethod
+    def replace_production(clouds, target_cloud):
+        content = yaml.load(open(clouds))
+        content['clouds']['production'] = content['clouds'][target_cloud]
+        del content['clouds'][target_cloud]
+        open(clouds, 'w').write(yaml.dump(content))
+
+    def clone(self, target_domain, target_cloud, clobber_cloud):
         config_base = os.path.dirname(self.config_dir)
         config_dir = f'{config_base}/{target_domain}'
-        sh.rsync('-a', '--delete',
-                 '--exclude=private-key.yml',
-                 '--exclude=hosts.yml',
-                 '--exclude=group_vars/all/domain.yml',
-                 '--exclude=group_vars/all/clouds.yml',
-                 '--exclude=group_vars/all/production_domain.yml',
-                 f'{self.config_dir}/', f'{config_dir}/')
-        clone_override_dir = f'{config_base}/clone-override'
-        if os.path.exists(clone_override_dir):
-            log.info(f'overriding with {clone_override_dir}')
-            sh.rsync('-av', '--checksum', f'{clone_override_dir}/', f'{config_dir}/')
-        if os.path.exists(f'{self.config_dir}.pass'):
-            shutil.copyfile(f'{self.config_dir}.pass', f'{config_dir}.pass')
-        all_dir = f'{config_dir}/inventory/group_vars/all'
-        if not os.path.exists(all_dir):
-            os.makedirs(all_dir)
-        clouds = f'{all_dir}/clouds.yml'
-        shutil.copyfile(target_clouds, clouds)
+        if not os.path.exists(config_dir) or clobber_cloud:
+            sh.rsync('-a', '--delete',
+                     '--exclude=private-key.yml',
+                     '--exclude=hosts.yml',
+                     '--exclude=group_vars/all/domain.yml',
+                     '--exclude=group_vars/all/production_domain.yml',
+                     f'{self.config_dir}/', f'{config_dir}/')
+            clone_override_dir = f'{config_base}/clone-override'
+            if os.path.exists(clone_override_dir):
+                log.info(f'overriding with {clone_override_dir}')
+                sh.rsync('-av', '--checksum', f'{clone_override_dir}/', f'{config_dir}/')
+            if os.path.exists(f'{self.config_dir}.pass'):
+                shutil.copyfile(f'{self.config_dir}.pass', f'{config_dir}.pass')
+            all_dir = f'{config_dir}/inventory/group_vars/all'
+            self.replace_production(f'{all_dir}/clouds.yml', target_cloud)
         kwargs = {
-            'domain': target_domain,
             'driver': self.args['driver'],
+            'domain': target_domain,
         }
         clone = Enough(config_dir, self.share_dir, **kwargs)
         return clone
@@ -149,19 +155,23 @@ class Enough(object):
             clone, snapshot)
         self._rsync_copy_host(from_ip, to_ip)
         self._clone_volume_from_snapshot_cleanup(clone, from_volume)
-        return to_volume
+        clone.openstack.o.volume.set('--name', snapshot, to_volume)
+
+    def create_volume_from_snapshot(self, snapshot, volume):
+        if not self.openstack.o.volume.list(
+                '-c', 'Name', '--format=value',
+                '--name', volume).strip() == volume:
+            self.openstack.o.volume.create('--snapshot', snapshot, volume)
 
     def _clone_volume_from_snapshot_body(self, clone, snapshot):
         #
         #         in the current region
         # Step 1: create volume from snapshot
         #
-        volume_name = re.sub('^\d\d\d\d-\d\d-\d\d-', '', snapshot)
+        volume_name = self.volume_from_snapshot(snapshot)
         from_volume = f'copy-from-{volume_name}'
-        if not self.openstack.o.volume.list(
-                '-c', 'Name', '--format=value',
-                '--name', from_volume).strip() == from_volume:
-            self.openstack.o.volume.create('--snapshot', snapshot, from_volume)
+        self.create_volume_from_snapshot(snapshot, from_volume)
+
         size = self.openstack.o.volume.show(
             '-c', 'size', '--format=value', from_volume).strip()
         #
@@ -193,22 +203,46 @@ class Enough(object):
         self.openstack.destroy_everything(None)
         sh.rm('-r', self.config_dir)
 
-    def restore(self):
-        return self.restore_remote(self.args['target_domain'],
-                                   self.args['target_clouds'],
-                                   self.args['name'])
+    def cli_clone_volume(self):
+        clone = self.clone(self.args['target_domain'],
+                           self.args['target_cloud'],
+                           self.args['clobber_cloud'])
+        self.clone_volume_from_snapshot(clone, self.args['name'])
+
+    @staticmethod
+    def volume_from_snapshot(snapshot):
+        return re.sub('^\d\d\d\d-\d\d-\d\d-', '', snapshot)
+
+    def host_from_snapshot(self, snapshot):
+        volume_name = self.volume_from_snapshot(snapshot)
+        return self.heat.host_from_volume(volume_name)
 
     def create_service_matching_snapshot(self, snapshot):
-        volume_name = re.sub('^\d\d\d\d-\d\d-\d\d-', '', snapshot)
-        host = self.heat.host_from_volume(volume_name)
+        host = self.host_from_snapshot(snapshot)
         service = self.service.service_from_host(host)
         self.set_args(name=service, playbook='enough-playbook.yml')
         self.service.create_or_update()
         return host
 
-    def restore_remote(self, domain, clouds, snapshot):
-        clone = self.clone(domain, clouds)
-        to_volume = self.clone_volume_from_snapshot(clone, snapshot)
+    def restore(self):
+        if self.args['target_domain']:
+            return self.restore_remote(self.args['target_domain'],
+                                       self.args['target_cloud'],
+                                       self.args['clobber_cloud'],
+                                       self.args['name'])
+        else:
+            return self.restore_local(self.args['name'])
+
+    def restore_local(self, snapshot):
+        host = self.host_from_snapshot(snapshot)
+        volume = snapshot
+        self.create_volume_from_snapshot(snapshot, volume)
+        self.openstack.replace_volume(host, volume, delete_volume=False)
+        return self
+
+    def restore_remote(self, domain, target_cloud, clobber_cloud, snapshot):
+        clone = self.clone(domain, target_cloud, clobber_cloud)
+        self.clone_volume_from_snapshot(clone, snapshot)
         host = clone.create_service_matching_snapshot(snapshot)
-        clone.openstack.replace_volume(host, to_volume)
+        clone.openstack.replace_volume(host, snapshot, delete_volume=True)
         return clone
