@@ -81,5 +81,5 @@ def docker_name():
 def openstack_name():
     prefix = 'enough_test_' + str(int(time.time()))
     yield prefix
-    o = OpenStack('.', 'inventory/group_vars/all/clouds.yml')
+    o = OpenStack('.')
     o.destroy_everything(prefix)
