@@ -20,6 +20,7 @@ def test_openstack_create_or_update(tmpdir, openstack_name, requests_mock):
         'playbook': os.path.abspath('tests/enough/common/test_common_service/enough-playbook.yml'),
         'domain': domain,
         'name': 'essential',
+        'cloud': 'production',
     })
     s.dotenough.set_certificate('ownca')
     s.dotenough.set_clouds_file('inventory/group_vars/all/clouds.yml')
@@ -37,13 +38,20 @@ def test_service_from_host():
     assert s.service_from_host('unknown-host') is None
 
 
+def test_set_service_info():
+    s = service.Service(settings.CONFIG_DIR, settings.SHARE_DIR)
+    assert 'bind-host' in s.service2hosts['bind']
+    assert len(s.service2hosts['bind']) > 0
+    assert ['bind-host'] == s.service2group['bind']
+
+
 def test_update_vpn_dependencies():
     s = service.Service(settings.CONFIG_DIR, settings.SHARE_DIR)
     assert s.hosts_with_internal_network(['bind-host']) == []
     assert 'website-host' not in s.service2hosts['openvpn']
     assert 'website-host' not in s.service2hosts['weblate']
     s.ansible.set_inventories(['tests/enough/common/test_common_service/vpn_inventory'])
-    s.set_service2hosts()
+    s.set_service_info()
     assert 'website-host' in s.service2hosts['openvpn']
     assert s.hosts_with_internal_network(['icinga-host']) == ['icinga-host']
     s.update_vpn_dependencies()
