@@ -2,6 +2,7 @@ import os
 import pytest
 import sh
 import shutil
+import yaml
 
 from enough import settings
 from enough.common import Enough
@@ -11,7 +12,6 @@ from enough.common.openstack import OpenStack, Stack
 @pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
                     reason='skip integration test')
 def test_clone_and_destroy(tmpdir):
-    clone_clouds = os.path.expanduser("~/.enough/dev/clone-clouds.yml")
     test_clouds = 'inventory/group_vars/all/clouds.yml'
 
     clone_override_dir = f'{tmpdir}/clone-override'
@@ -36,7 +36,7 @@ def test_clone_and_destroy(tmpdir):
     assert original.openstack.o.server.list().strip() == ''
 
     clone_domain = 'clone.com'
-    clone = original.clone(clone_domain, clone_clouds)
+    clone = original.clone(clone_domain, 'clone', True)
 
     assert password == open(f'{clone.config_dir}.pass').read()
     assert 'REPLACED CONTENT' == open(
@@ -48,7 +48,9 @@ def test_clone_and_destroy(tmpdir):
     assert not os.path.exists(clone.config_dir)
 
 
-def create_enough(tmpdir, clouds, dotenough, **kwargs):
+def create_enough(tmpdir, dotenough, **kwargs):
+    clouds = 'inventory/group_vars/all/clouds.yml'
+
     enough_domain = 'enough.com'
     enough_config_dir = f'{tmpdir}/{enough_domain}'
     shutil.copytree(f'tests/enough/common/test_init/{dotenough}',
@@ -67,13 +69,30 @@ def create_enough(tmpdir, clouds, dotenough, **kwargs):
     return enough
 
 
-def create_and_clone_server_and_volume(tmpdir, clone_clouds, test_clouds):
-    original = create_enough(tmpdir, test_clouds, 'backup')
+@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
+                    reason='skip integration test')
+def test_clone_clobber(tmpdir):
+    original = create_enough(tmpdir, 'copy')
+    clone_domain = 'clone.com'
+    clone = original.clone(clone_domain, 'clone', False)
+    assert os.path.exists(f'{clone.config_dir}')
+    stone = f'{clone.config_dir}/STONE'
+    open(stone, 'w').write('INSIDE')
+
+    assert os.path.exists(stone)
+    clone = original.clone(clone_domain, 'clone', False)
+    assert os.path.exists(stone)
+    clone = original.clone(clone_domain, 'clone', True)
+    assert not os.path.exists(stone)
+
+
+def create_and_clone_server_and_volume(tmpdir):
+    original = create_enough(tmpdir, 'backup')
     original.set_args(name='sample', playbook='enough-playbook.yml')
     original.service.create_or_update()
 
     clone_domain = 'clone.com'
-    clone = original.clone(clone_domain, clone_clouds)
+    clone = original.clone(clone_domain, 'clone', True)
     clone.set_args(name='sample', playbook='enough-playbook.yml')
     clone.service.create_or_update()
 
@@ -83,18 +102,15 @@ def create_and_clone_server_and_volume(tmpdir, clone_clouds, test_clouds):
 @pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
                     reason='skip integration test')
 def test_clone_create_service(tmpdir):
-    clone_clouds = os.path.expanduser("~/.enough/dev/clone-clouds.yml")
-    test_clouds = 'inventory/group_vars/all/clouds.yml'
-
     try:
-        (original, clone) = create_and_clone_server_and_volume(tmpdir, clone_clouds, test_clouds)
+        (original, clone) = create_and_clone_server_and_volume(tmpdir)
         assert 'sample-host' in original.openstack.o.server.list()
         assert 'sample-volume' in original.openstack.o.volume.list()
         assert 'sample-host' in clone.openstack.o.server.list()
         assert 'sample-volume' in clone.openstack.o.volume.list()
     finally:
-        for clouds in (test_clouds, clone_clouds):
-            o = OpenStack(settings.CONFIG_DIR, clouds)
+        for cloud in ('production', 'clone'):
+            o = OpenStack(settings.CONFIG_DIR, cloud=cloud)
             # comment out the following line to re-use the content of the regions and save time
             o.destroy_everything(None)
 
@@ -102,10 +118,8 @@ def test_clone_create_service(tmpdir):
 @pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
                     reason='skip integration test')
 def test_create_copy_host(tmpdir):
-    clouds = 'inventory/group_vars/all/clouds.yml'
-
     try:
-        enough = create_enough(tmpdir, clouds, 'copy')
+        enough = create_enough(tmpdir, 'copy')
         if 'copy-volume' not in enough.openstack.o.volume.list():
             enough.openstack.o.volume.create('--size', '1', 'copy-volume')
         ip = enough.create_copy_host('copy-from-host', 'some-volume', 'copy-volume')
@@ -116,7 +130,29 @@ def test_create_copy_host(tmpdir):
                    '-i', enough.dotenough.private_key(), f'root@{ip}', 'mountpoint', '/srv').strip()
         assert r == '/srv is a mountpoint'
     finally:
-        o = OpenStack(settings.CONFIG_DIR, clouds)
+        o = OpenStack(settings.CONFIG_DIR)
+        # comment out the following line to re-use the content of the regions and save time
+        o.destroy_everything(None)
+
+
+@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
+                    reason='skip integration test')
+def test_create_volume_from_snapshot(tmpdir):
+    try:
+        e = create_enough(tmpdir, 'backup')
+        e.set_args(name='sample', playbook='enough-playbook.yml')
+        e.service.create_or_update()
+        e.openstack.backup_create(['sample-volume'])
+        snapshot = f'{e.openstack.backup_date()}-sample-volume'
+        assert snapshot in e.openstack.o.volume.snapshot.list()
+
+        volume = 'test-volume'
+        e.create_volume_from_snapshot(snapshot, volume)
+
+        assert volume in e.openstack.o.volume.list()
+
+    finally:
+        o = OpenStack(settings.CONFIG_DIR)
         # comment out the following line to re-use the content of the regions and save time
         o.destroy_everything(None)
 
@@ -124,11 +160,8 @@ def test_create_copy_host(tmpdir):
 @pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
                     reason='skip integration test')
 def test_clone_volume_from_snapshot(tmpdir):
-    clone_clouds = os.path.expanduser("~/.enough/dev/clone-clouds.yml")
-    test_clouds = 'inventory/group_vars/all/clouds.yml'
-
     try:
-        original = create_enough(tmpdir, test_clouds, 'backup')
+        original = create_enough(tmpdir, 'backup')
         original.set_args(name='sample', playbook='enough-playbook.yml')
         original.service.create_or_update()
         ip = original.hosts.load().get_ip('sample-host')
@@ -141,7 +174,7 @@ def test_clone_volume_from_snapshot(tmpdir):
         assert snapshot in original.openstack.o.volume.snapshot.list()
 
         clone_domain = 'clone.com'
-        clone = original.clone(clone_domain, clone_clouds)
+        clone = original.clone(clone_domain, 'clone', True)
 
         (from_ip, to_ip, from_volume, to_volume) = original._clone_volume_from_snapshot_body(
             clone, snapshot)
@@ -160,8 +193,8 @@ def test_clone_volume_from_snapshot(tmpdir):
         original._clone_volume_from_snapshot_cleanup(clone, from_volume)
 
     finally:
-        for clouds in (test_clouds, clone_clouds):
-            o = OpenStack(settings.CONFIG_DIR, clouds)
+        for cloud in ('production', 'clone'):
+            o = OpenStack(settings.CONFIG_DIR, cloud=cloud)
             # comment out the following line to re-use the content of the regions and save time
             o.destroy_everything(None)
 
@@ -169,16 +202,14 @@ def test_clone_volume_from_snapshot(tmpdir):
 @pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
                     reason='skip integration test')
 def test_create_service_matching_snapshot(tmpdir):
-    test_clouds = 'inventory/group_vars/all/clouds.yml'
-
     try:
-        enough = create_enough(tmpdir, test_clouds, 'backup')
+        enough = create_enough(tmpdir, 'backup')
         host = enough.create_service_matching_snapshot('2020-02-20-sample-volume')
         assert host == 'sample-host'
         assert 'sample-volume' in enough.openstack.o.volume.list()
 
     finally:
-        o = OpenStack(settings.CONFIG_DIR, test_clouds)
+        o = OpenStack(settings.CONFIG_DIR)
         # comment out the following line to re-use the content of the regions and save time
         o.destroy_everything(None)
 
@@ -186,11 +217,8 @@ def test_create_service_matching_snapshot(tmpdir):
 @pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
                     reason='skip integration test')
 def test_restore_remote(tmpdir):
-    clone_clouds = os.path.expanduser("~/.enough/dev/clone-clouds.yml")
-    test_clouds = 'inventory/group_vars/all/clouds.yml'
-
     try:
-        original = create_enough(tmpdir, test_clouds, 'backup')
+        original = create_enough(tmpdir, 'backup')
         original.set_args(name='sample', playbook='enough-playbook.yml')
         original.service.create_or_update()
         ip = original.hosts.load().get_ip('sample-host')
@@ -201,7 +229,7 @@ def test_restore_remote(tmpdir):
         snapshot = f'{original.openstack.backup_date()}-sample-volume'
         assert snapshot in original.openstack.o.volume.snapshot.list()
 
-        clone = original.restore_remote('test.com', clone_clouds, snapshot)
+        clone = original.restore_remote('test.com', 'clone', True, snapshot)
         assert 'sample-volume' in clone.openstack.o.volume.list()
         hosts = clone.hosts.load()
         Stack.wait_for_ssh(hosts.get_ip('sample-host'), hosts.get_port('sample-host'))
@@ -210,19 +238,47 @@ def test_restore_remote(tmpdir):
                       'test', '-e', '/srv/STONE').exit_code == 0
 
     finally:
-        for clouds in (test_clouds, clone_clouds):
-            o = OpenStack(settings.CONFIG_DIR, clouds)
+        for cloud in ('production', 'clone'):
+            o = OpenStack(settings.CONFIG_DIR, cloud=cloud)
             # comment out the following line to re-use the content of the regions and save time
             o.destroy_everything(None)
 
 
 @pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
                     reason='skip integration test')
-def test_create_missings(tmpdir):
-    test_clouds = 'inventory/group_vars/all/clouds.yml'
-
+def test_restore_local(tmpdir):
     try:
-        enough = create_enough(tmpdir, test_clouds, 'create-missings')
+        e = create_enough(tmpdir, 'backup')
+        e.set_args(name='sample', playbook='enough-playbook.yml')
+        e.service.create_or_update()
+        ip = e.hosts.load().get_ip('sample-host')
+
+        sh.ssh('-oStrictHostKeyChecking=no',
+               '-i', e.dotenough.private_key(), f'debian@{ip}', 'touch', '/srv/STONE')
+        e.openstack.backup_create(['sample-volume'])
+        sh.ssh('-oStrictHostKeyChecking=no',
+               '-i', e.dotenough.private_key(), f'debian@{ip}', 'rm', '/srv/STONE')
+        snapshot = f'{e.openstack.backup_date()}-sample-volume'
+        assert snapshot in e.openstack.o.volume.snapshot.list()
+
+        assert e == e.restore_local(snapshot)
+        assert 'sample-volume' in e.openstack.o.volume.list()
+        Stack.wait_for_ssh(e.hosts.get_ip('sample-host'), e.hosts.get_port('sample-host'))
+        assert sh.ssh('-oStrictHostKeyChecking=no',
+                      '-i', e.dotenough.private_key(), f'debian@{ip}',
+                      'test', '-e', '/srv/STONE').exit_code == 0
+
+    finally:
+        o = OpenStack(settings.CONFIG_DIR)
+        # comment out the following line to re-use the content of the regions and save time
+        o.destroy_everything(None)
+
+
+@pytest.mark.skipif('SKIP_OPENSTACK_INTEGRATION_TESTS' in os.environ,
+                    reason='skip integration test')
+def test_create_missings(tmpdir):
+    try:
+        enough = create_enough(tmpdir, 'create-missings')
         r = enough.create_missings(['bind-host'])
         assert 'bind-host' in r
         internal_dns = enough.openstack.o.subnet.show(
@@ -230,6 +286,35 @@ def test_create_missings(tmpdir):
         bind_internal_ip = enough.openstack.server_ip_in_network('bind-host', 'internal')
         assert bind_internal_ip == internal_dns
     finally:
-        o = OpenStack(settings.CONFIG_DIR, test_clouds)
+        o = OpenStack(settings.CONFIG_DIR)
         # comment out the following line to re-use the content of the regions and save time
         o.destroy_everything(None)
+
+
+def test_replace_production(tmpdir):
+    target_cloud = 'clone'
+    clouds = {
+        'clouds': {
+            target_cloud: 'CLONE',
+            'production': 'PRODUCTION',
+        }
+    }
+    clouds_path = f'{tmpdir}/clouds.yml'
+    open(clouds_path, 'w').write(yaml.dump(clouds))
+    Enough.replace_production(clouds_path, target_cloud)
+    clouds = yaml.load(open(clouds_path))
+    assert clouds['clouds']['production'] == 'CLONE'
+    assert target_cloud not in clouds['clouds']
+
+
+def test_volume_from_snapshot():
+    volume = 'NAME'
+    snapshot = f'2010-07-08-{volume}'
+    assert Enough.volume_from_snapshot(snapshot) == volume
+
+
+def test_host_from_snapshot(tmpdir):
+    e = create_enough(tmpdir, 'backup')
+    host = 'sample-host'
+    snapshot = f'2010-07-08-sample-volume'
+    assert e.host_from_snapshot(snapshot) == host
